import { Router, RouterOptions } from "express";
import { RequestHandler } from "../lib/helpers";
import { RequestMethods } from "./enums";
/**
 *
 * Controller Decorator
 * This decorator allows you to create Controller Classes for your Application.
 *
 */
export interface ControllerInterface {
    prefix: string;
    router: Router;
    constructor: new () => any;
    options: ControllerOptions;
}
export interface ControllerOptions {
    childs?: (new () => any)[];
    isChild?: true;
    isOpen?: true;
    routerOptions?: RouterOptions;
}
export declare const Controller: (prefix?: string, options?: ControllerOptions | undefined) => (constructor: new () => any) => void;
export interface ChildControllerOptions {
    childs?: (new () => any)[];
    routerOptions?: RouterOptions;
}
export declare const ChildController: (prefix?: string, options?: ControllerOptions | undefined) => (constructor: new () => any) => void;
/**
 *
 * Routes Decorator
 * This decorator allows you to create routes under Controller.
 *
 */
export interface RouteHandler {
    method: RequestMethods;
    path: string;
    name: string;
    description: string;
    handler: RequestHandler;
}
export interface RouteHandlerDescriptor extends PropertyDescriptor {
    value?: RequestHandler;
}
export declare const Get: (path?: string, description?: string) => (target: any, key: string, desc: RouteHandlerDescriptor) => void;
export declare const Post: (path?: string, description?: string) => (target: any, key: string, desc: RouteHandlerDescriptor) => void;
export declare const Patch: (path?: string, description?: string) => (target: any, key: string, desc: RouteHandlerDescriptor) => void;
export declare const Put: (path?: string, description?: string) => (target: any, key: string, desc: RouteHandlerDescriptor) => void;
export declare const Delete: (path?: string, description?: string) => (target: any, key: string, desc: RouteHandlerDescriptor) => void;
export declare const Options: (path?: string, description?: string) => (target: any, key: string, desc: RouteHandlerDescriptor) => void;
/**
 *
 * Middleware Decorator
 * This decorator allows you to add middlewares on the route.
 *
 */
export interface UseDecorator {
    (constructor: new () => any): void;
    (target: any, key: string, desc: RouteHandlerDescriptor): void;
}
export declare const Use: (...middlewares: RequestHandler[]) => UseDecorator;
