import { HTTPBase } from "./http-base";
import { EpicServerException } from "./exceptions";
import { RequestErrorHandler } from "./helpers";
/**
 *
 * Create Epic Application
 * This class will create an instance of Application framework. Default framework is express.
 *
 */
export interface EpicApplicationInterface {
    _beforeInit?: (Server: HTTPBase) => Promise<any> | any;
    _onInit?: (Server: HTTPBase) => Promise<any> | any;
    _onRouteError?: RequestErrorHandler;
    _onError?: (error: EpicServerException, _Server: HTTPBase) => Promise<any> | any;
}
export declare class EpicApplication {
    Framework: import("express-serve-static-core").Express;
    constructor(options?: EpicApplicationInterface, Framework?: import("express-serve-static-core").Express);
    /**
     * Before the server initializes
     * This method let you configure the middleware required by your application to works.
     *
     */
    _beforeInit: (_Server: HTTPBase) => Promise<any> | any;
    /**
     * When the server starts listening
     * Do something when the server starts listening...
     *
     */
    _onInit: (Server: HTTPBase) => Promise<any> | any;
    /**
     * When the server starts listening
     * Do something when the server starts listening...
     *
     */
    _onRouteError: RequestErrorHandler;
    /**
     *
     * Error occured while starting the server
     * Do something if there was an error while starting the server.
     *
     */
    _onError: (error: EpicServerException, _Server: HTTPBase) => Promise<any> | any;
}
