"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTTPBase = void 0;
const http_errors_1 = __importDefault(require("http-errors"));
const express_request_id_1 = __importDefault(require("express-request-id"));
const globals_1 = require("./globals");
const exceptions_1 = require("./exceptions");
const helpers_1 = require("./helpers");
class HTTPBase {
    constructor(Application, Server, ServerDebug) {
        this.Application = Application;
        this.Server = Server;
        this.ServerDebug = ServerDebug;
    }
    listen(port) {
        return __awaiter(this, void 0, void 0, function* () {
            // Normalize Port
            this.Port = this.normalizePort(port);
            // Add Request ID
            this.Application.Framework.use(express_request_id_1.default());
            // Before Start Listening to the Server
            yield this.Application._beforeInit(this);
            // Add Routes
            this.Application.Framework.use(globals_1.EpicGlobals.getRouter());
            // Add Routes Error Handler
            this.Application.Framework.use((_req, _res, next) => next(http_errors_1.default(404))).use(helpers_1.EpicHelpers.requestErrorHandler(this.Application._onRouteError));
            // Listen on provided port, on all network interfaces.
            this.Server.listen(this.Port);
            // Event listener for HTTP/HTTPS Server.
            this.Server.on("listening", () => __awaiter(this, void 0, void 0, function* () {
                const Address = this.Server.address();
                const Bind = typeof Address === "string"
                    ? "Pipe " + Address
                    : "Port " + (Address === null || Address === void 0 ? void 0 : Address.port);
                this.ServerDebug("Listening on " + Bind);
                yield this.Application._onInit(this);
            }));
            // Event listener for HTTP server "error" event.
            this.Server.on("error", (error) => __awaiter(this, void 0, void 0, function* () {
                try {
                    if (error.syscall !== "listen")
                        throw error;
                    // Get Port Binding
                    const Bind = typeof this.Port === "string"
                        ? "Pipe " + this.Port
                        : "Port " + this.Port;
                    // Handle specific listen errors with friendly messages
                    switch (error.code) {
                        case "EACCES":
                            throw new Error(Bind + " requires elevated privileges!");
                        case "EADDRINUSE":
                            throw new Error(Bind + " is already in use!");
                        default:
                            throw error;
                    }
                }
                catch (error) {
                    // Error Occured While Starting the Server
                    yield this.Application._onError(new exceptions_1.EpicServerException(error.message || error), this);
                }
            }));
        });
    }
    /**
     *
     * Utility Functions
     * These utility functions are used to safely start the server.
     *
     */
    // Normalize Server Port
    normalizePort(port) {
        // Resolve Port
        if (typeof port == "string")
            port = parseInt(port, 10);
        // Validate Port
        if (isNaN(port))
            return port;
        else if (port >= 0)
            return port;
        else
            throw new exceptions_1.EpicServerException(`Invalid Port '${port}' has been provided!`);
    }
}
exports.HTTPBase = HTTPBase;
