import http from "http";
import https from "https";
import Debug from "debug";
import { HTTPBase } from "./http-base";
import { EpicApplication } from "./application";

/**
 *
 * HTTP Server
 * This class will create an instance of HTTP Server.
 *
 */
export class HTTP extends HTTPBase {
  constructor(
    Application: EpicApplication,
    ServerOptions?: Parameters<typeof http.createServer>[0],
    Debugger = Debug("Epic:Server")
  ) {
    super(
      Application,
      http.createServer(ServerOptions || {}, Application.Framework),
      Debugger
    );
  }
}

/**
 *
 * HTTPS Server
 * This class will create an instance of HTTPS Server.
 *
 */
export class HTTPS extends HTTPBase {
  public Server!: https.Server;

  constructor(
    Application: EpicApplication,
    ServerOptions?: Parameters<typeof https.createServer>[0],
    Debugger = Debug("Epic:Server")
  ) {
    super(
      Application,
      https.createServer(ServerOptions || {}, Application.Framework),
      Debugger
    );
  }
}
